﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Blazorise;
using Blazorise.DataGrid;
using Microsoft.AspNetCore.Components;
using UserService.DatabaseLayer.Repositories;
using UserService.Infrastructure;
using UserService.Infrastructure.DataModels;

namespace UserService.Pages
{
    public class SecurityGroupsBase : MembersBase<SecurityGroup>
    {
        [Inject] private ISecurityGroupsRepository SecurityGroupsRepository { get; set; } = null!;

        [Inject] private IUsersRepository UsersRepository { get; set; } = null!;

        [Inject] private IOrganizationUnitsRepository OrganizationUnitsRepository { get; set; } = null!;

        // reference to the modal component
        protected Modal ModalRef { get; set; } = null!;

        protected IEnumerable<Member>? AvailableMembers { get; set; }

        protected override async Task OnInitializedAsync()
        {
            Members = await SecurityGroupsRepository.GetAllAsync().ConfigureAwait(false);
            OrganizationUnits = await OrganizationUnitsRepository.GetAllAsync().ConfigureAwait(false);
        }

        /// <inheritdoc />
        protected override async Task RowInsertingCallback(
            CancellableRowChange<SecurityGroup, Dictionary<string, object>> arg)
        {
        }

        /// <inheritdoc />
        protected override async Task RowInsertedCallback(SavedRowItem<SecurityGroup, Dictionary<string, object>> arg)
        {
        }

        /// <inheritdoc />
        protected override async Task RowDeletingCallback(CancellableRowChange<SecurityGroup> arg)
        {
            if (arg == null) throw new ArgumentNullException(nameof(arg));
            var confirmed = await JsRuntime.ConfirmAsync($"You are about to delete the security group {arg.Item.CommonName}. Are you sure?")
                .ConfigureAwait(false);
            arg.Cancel = !confirmed;
        }

        /// <inheritdoc />
        protected override async Task RowUpdatingCallback(CancellableRowChange<SecurityGroup, Dictionary<string, object>> arg)
        {
            if (arg == null) throw new ArgumentNullException(nameof(arg));
            var securityGroup = arg.Item;
            securityGroup.MapFields(arg.Values);
            securityGroup.Parent =
                OrganizationUnits?.FirstOrDefault(x => x.Id == (Guid?)arg.Values[nameof(Node.ParentId)]);
            var result = await SecurityGroupsRepository.UpdateAsync(securityGroup).ConfigureAwait(false);
            arg.Cancel = !result;
        }

        protected async Task MySearchHandler(object arg)
        {
            if (arg is Member addedMember)
            {
                SelectedMember = addedMember;
            }

        }

        protected async Task OnSearchChanged(string arg)
        {
            AvailableMembers = await GetAvailableMembers(arg).ConfigureAwait(false);
        }

        private async Task<IReadOnlyList<Member>?> GetAvailableMembers(string arg)
        {
            var result = new List<Member>();
            if (string.IsNullOrWhiteSpace(arg)) return null;
            var users = await UsersRepository.GetAllAsync().ConfigureAwait(false);
            var securityGroups = await SecurityGroupsRepository.GetAllAsync().ConfigureAwait(false);
            result.AddRange(users.Where(x =>
                x.CommonName.StartsWith(arg, StringComparison.InvariantCultureIgnoreCase) ||
                x.FullName.StartsWith(arg, StringComparison.InvariantCultureIgnoreCase)));
            result.AddRange(securityGroups
                .Where(x => x.CommonName.StartsWith(arg, StringComparison.InvariantCultureIgnoreCase))
                .Where(x => x.Id != SelectedSecurityGroup?.Id));
            return result.Count == 0 ? null : result;
        }


        protected async Task AddToListAsync()
        {
            if (SelectedMember is null) return;
            if (SelectedSecurityGroup is null) return;
            if (!SelectedSecurityGroup.Members.Add(new MembersMember
            {
                MemberId = SelectedSecurityGroup.Id,
                Member = SelectedSecurityGroup,
                AttachedMemberId = SelectedMember.Id,
                AttachedMember = SelectedMember
            }))
            {
                await JsRuntime.AlertAsync("User already part of the list.").ConfigureAwait(false);
            }

            SelectedMember = default;
        }

        protected void OnButtonClicked(SecurityGroup securityGroup)
        {
            SelectedSecurityGroup = securityGroup;
            ModalRef.Show();
        }

        public SecurityGroup? SelectedSecurityGroup { get; set; }

        /// <inheritdoc />
        protected override bool OnCustomFilter(SecurityGroup model)
        {
            if (model == null) throw new ArgumentNullException(nameof(model));
            // We want to accept empty value as valid or otherwise
            // datagrid will not show anything.
            if (string.IsNullOrEmpty(CustomFilterValue)) return true;

            return model.CommonName.Contains(CustomFilterValue, StringComparison.OrdinalIgnoreCase);
        }

        protected override Task RowDeletedCallback(SecurityGroup item) => SecurityGroupsRepository.DeleteAsync(item);

        public Member? SelectedMember { get; set; }

        protected async Task HideModalAsync(bool save)
        {
            if (save && SelectedSecurityGroup != null) await SecurityGroupsRepository.UpdateAsync(SelectedSecurityGroup).ConfigureAwait(false);
            ModalRef.Hide();
        }
    }
}