﻿using System;
using Microsoft.AspNetCore.Components;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Blazorise;
using Blazorise.DataGrid;
using UserService.DatabaseLayer.Repositories;
using UserService.Infrastructure.DataModels;
using UserService.Infrastructure;

namespace UserService.Pages
{
    public class UsersBase : MembersBase<User>
    {
        [Inject] private IUsersRepository UsersRepository { get; set; } = null!;
        [Inject] private IOrganizationUnitsRepository OrganizationUnitsRepository { get; set; } = null!;

        protected override async Task OnInitializedAsync()
        {
            Members = await UsersRepository.GetAllAsync().ConfigureAwait(false);
            OrganizationUnits = await OrganizationUnitsRepository.GetAllAsync().ConfigureAwait(false);
        }

        protected override async Task RowInsertingCallback(CancellableRowChange<User, Dictionary<string, object>> arg)
        {
            if (arg is null) throw new ArgumentNullException(nameof(arg));
            var mailValidation = Validators.ValidateEmail(arg.Values[nameof(User.EMail)]?.ToString());
            var commonNameValidation = Validators.ValidateCommonName(arg.Values[nameof(User.CommonName)]?.ToString(),
                Members ?? Enumerable.Empty<User>());
            if (mailValidation == true && commonNameValidation == true) return;

            await JsRuntime.AlertAsync("User could not be added").ConfigureAwait(false);
            arg.Cancel = true;
        }

        protected override async Task RowInsertedCallback(SavedRowItem<User, Dictionary<string, object>> arg)
        {
            if (arg is null) throw new ArgumentNullException(nameof(arg));
            var user = arg.Item;
            await UsersRepository.AddAsync(user).ConfigureAwait(false);
            user.Parent = OrganizationUnits?.FirstOrDefault(x => x.Id == user.ParentId);
        }

        protected override async Task RowDeletingCallback(CancellableRowChange<User> arg)
        {
            if (arg == null) throw new ArgumentNullException(nameof(arg));
            var confirmed = await JsRuntime.ConfirmAsync($"You are about to delete the user {arg.Item.FullName}. Are you sure?").ConfigureAwait(false);
            arg.Cancel = !confirmed;
        }

        protected override Task RowDeletedCallback(User user) => UsersRepository.DeleteAsync(user);


        protected static void ValidateEmail(ValidatorEventArgs e)
        {
            if (e == null) throw new ArgumentNullException(nameof(e));
            var email = e.Value?.ToString();
            var validationResult = Validators.ValidateEmail(email);

            e.Status = validationResult switch
            {
                null => ValidationStatus.None,
                false => ValidationStatus.Error,
                _ => ValidationStatus.Success
            };
        }

        protected override async Task RowUpdatingCallback(CancellableRowChange<User, Dictionary<string, object>> arg)
        {
            if (arg == null) throw new ArgumentNullException(nameof(arg));
            var user = arg.Item;
            user.MapFields(arg.Values);
            user.Parent = OrganizationUnits?.FirstOrDefault(x => x.Id == (Guid?)arg.Values[nameof(Node.ParentId)]);
            var result = await UsersRepository.UpdateAsync(user).ConfigureAwait(false);
            arg.Cancel = !result;
        }

        protected override bool OnCustomFilter(User model)
        {
            if (model == null) throw new ArgumentNullException(nameof(model));
            // We want to accept empty value as valid or otherwise
            // datagrid will not show anything.
            if (string.IsNullOrEmpty(CustomFilterValue) || CustomFilterValue.Length < 3) return true;

            return
                model.FirstName?.Contains(CustomFilterValue, StringComparison.OrdinalIgnoreCase) == true
                || model.LastName?.Contains(CustomFilterValue, StringComparison.OrdinalIgnoreCase) == true
                || model.CommonName.Contains(CustomFilterValue, StringComparison.OrdinalIgnoreCase);
        }
    }
}